unit UOrderFieldsDlg;

interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls,
  Buttons, DB, DbTables;

type
  TOrderFieldsDlg = class(TForm)
    OKBtn: TButton;
    CancelBtn: TButton;
    SrcList: TListBox;
    DstList: TListBox;
    SrcLabel: TLabel;
    DstLabel: TLabel;
    IncludeBtn: TSpeedButton;
    IncAllBtn: TSpeedButton;
    ExcludeBtn: TSpeedButton;
    ExAllBtn: TSpeedButton;
    chkPseudoIndexes: TCheckBox;
    procedure IncludeBtnClick(Sender: TObject);
    procedure ExcludeBtnClick(Sender: TObject);
    procedure IncAllBtnClick(Sender: TObject);
    procedure ExcAllBtnClick(Sender: TObject);
    procedure MoveSelected(List: TCustomListBox; Items: TStrings);
    procedure SetItem(List: TListBox; Index: Integer);
    function GetFirstSelection(List: TCustomListBox): Integer;
    procedure SetButtons;
    procedure FormShow(Sender: TObject);
  private
    { Private declarations }
    theTable: TTable;
  public
    { Public declarations }
    property Table: TTable read theTable write theTable;
    class procedure SetOrdering(aTable: TTable);
  end;

var
  OrderFieldsDlg: TOrderFieldsDlg;

implementation

{$R *.DFM}

procedure TOrderFieldsDlg.IncludeBtnClick(Sender: TObject);
var
  Index: Integer;
begin
  Index := GetFirstSelection(SrcList);
  MoveSelected(SrcList, DstList.Items);
  SetItem(SrcList, Index);
end;

procedure TOrderFieldsDlg.ExcludeBtnClick(Sender: TObject);
var
  Index: Integer;
begin
  Index := GetFirstSelection(DstList);
  MoveSelected(DstList, SrcList.Items);
  SetItem(DstList, Index);
end;

procedure TOrderFieldsDlg.IncAllBtnClick(Sender: TObject);
var
  I: Integer;
begin
  for I := 0 to SrcList.Items.Count - 1 do
    DstList.Items.AddObject(SrcList.Items[I], 
      SrcList.Items.Objects[I]);
  SrcList.Items.Clear;
  SetItem(SrcList, 0);
end;

procedure TOrderFieldsDlg.ExcAllBtnClick(Sender: TObject);
var
  I: Integer;
begin
  for I := 0 to DstList.Items.Count - 1 do
    SrcList.Items.AddObject(DstList.Items[I], DstList.Items.Objects[I]);
  DstList.Items.Clear;
  SetItem(DstList, 0);
end;

procedure TOrderFieldsDlg.MoveSelected(List: TCustomListBox; Items: TStrings);
var
  I: Integer;
begin
  for I := List.Items.Count - 1 downto 0 do
    if List.Selected[I] then
    begin
      Items.AddObject(List.Items[I], List.Items.Objects[I]);
      List.Items.Delete(I);
    end;
end;

procedure TOrderFieldsDlg.SetButtons;
var
  SrcEmpty, DstEmpty: Boolean;
begin
  SrcEmpty := SrcList.Items.Count = 0;
  DstEmpty := DstList.Items.Count = 0;
  IncludeBtn.Enabled := not SrcEmpty;
  IncAllBtn.Enabled := not SrcEmpty;
  ExcludeBtn.Enabled := not DstEmpty;
  ExAllBtn.Enabled := not DstEmpty;
end;

function TOrderFieldsDlg.GetFirstSelection(List: TCustomListBox): Integer;
begin
  for Result := 0 to List.Items.Count - 1 do
    if List.Selected[Result] then Exit;
  Result := LB_ERR;
end;

procedure TOrderFieldsDlg.SetItem(List: TListBox; Index: Integer);
var
  MaxIndex: Integer;
begin
  with List do
  begin
    SetFocus;
    MaxIndex := List.Items.Count - 1;
    if Index = LB_ERR then Index := 0
    else if Index > MaxIndex then Index := MaxIndex;
    Selected[Index] := True;
  end;
  SetButtons;
end;

class procedure TOrderFieldsDlg.SetOrdering(aTable: TTable);
var
  i: Integer;
  indexExp: String;
begin
  with Create(nil) do
    try
      theTable := aTable;
      if ShowModal = mrOK then begin
        indexExp := '';
        if chkPseudoIndexes.Checked then begin
          for i := 0 to DstList.Items.Count - 1 do
            indexExp := indexExp + '@' + DstList.Items[i];
          table.IndexName := indexExp;
        end
        else begin
          for i := 0 to DstList.Items.Count - 1 do begin
            if ( i <> 0 ) then
              indexExp := indexExp + ';';
            indexExp := indexExp + DstList.Items[i];
          end;
          table.IndexFieldNames := indexExp;
        end;
      end;
    finally
      Free;
    end;
end;

procedure TOrderFieldsDlg.FormShow(Sender: TObject);
var
  i: Integer;
begin
  if theTable <> nil then
    with theTable do
      for i := 0 to FieldCount - 1 do
        SrcList.Items.Add(Fields[i].FieldName);
end;

end.
